@@ -61,7 +61,7 @@
  *     </thead>
  *     <tbody>
  *       <row>
- *         <entry><filename>/var/lib/udisks2/mounted-fs</filename></entry>
+ *         <entry><filename>/run/udisks2/mounted-fs</filename></entry>
  *         <entry>
  *           A serialized 'a{sa{sv}}' #GVariant mapping from the
  *           mount point (e.g. <filename>/media/EOS_DIGITAL</filename>) into a set of details.
@@ -110,6 +110,17 @@
  *           of the user who set up the loop device.
  *         </entry>
  *       </row>
+ *       <row>
+ *         <entry><filename>/run/udisks2/mdraid</filename></entry>
+ *         <entry>
+ *           A serialized 'a{ta{sv}}' #GVariant mapping from the
+ *           #dev_t of the raid device (e.g. <filename>/dev/md127</filename>) into a set of details.
+ *           Known details include
+ *           <literal>started-by-uid</literal>
+ *           (of type <link linkend="G-VARIANT-TYPE-UINT32:CAPS">'u'</link>) that is the #uid_t
+ *           of the user who started the array.
+ *         </entry>
+ *       </row>
  *     </tbody>
  *   </tgroup>
  * </table>
@@ -173,6 +184,10 @@ static void udisks_cleanup_check_loop (UDisksCleanup *cleanup,
                                        gboolean       check_only,
                                        GArray        *devs_to_clean);
 
+static void udisks_cleanup_check_mdraid (UDisksCleanup *cleanup,
+                                         gboolean       check_only,
+                                         GArray        *devs_to_clean);
+
 G_DEFINE_TYPE (UDisksCleanup, udisks_cleanup, G_TYPE_OBJECT);
 
 
@@ -411,6 +426,10 @@ udisks_cleanup_check_in_thread (UDisksCleanup *cleanup)
                              TRUE, /* check_only */
                              devs_to_clean);
 
+  udisks_cleanup_check_mdraid (cleanup,
+                               TRUE, /* check_only */
+                               devs_to_clean);
+
   /* Then go through all mounted filesystems and pass the
    * devices that we intend to clean...
    */
@@ -426,6 +445,10 @@ udisks_cleanup_check_in_thread (UDisksCleanup *cleanup)
                              FALSE, /* check_only */
                              NULL);
 
+  udisks_cleanup_check_mdraid (cleanup,
+                               FALSE, /* check_only */
+                               NULL);
+
   g_array_unref (devs_to_clean);
 
   udisks_info ("Cleanup check end");
@@ -677,6 +700,7 @@ udisks_cleanup_check_mounted_fs_entry (UDisksCleanup  *cleanup,
           /* right now -l is the only way to "force unmount" file systems... */
           if (!udisks_daemon_launch_spawned_job_sync (cleanup->daemon,
                                                       NULL, /* UDisksObject */
+                                                      "cleanup", 0, /* StartedByUID */
                                                       NULL, /* GCancellable */
                                                       0,    /* uid_t run_as_uid */
                                                       0,    /* uid_t run_as_euid */
@@ -823,7 +847,7 @@ udisks_cleanup_check_mounted_fs (UDisksCleanup *cleanup,
  * @fstab_mount: %TRUE if the device was mounted via /etc/fstab.
  *
  * Adds a new entry to the
- * <filename>/var/lib/udisks2/mounted-fs</filename> file.
+ * <filename>/run/udisks2/mounted-fs</filename> file.
  */
 void
 udisks_cleanup_add_mounted_fs (UDisksCleanup  *cleanup,
@@ -869,7 +893,18 @@ udisks_cleanup_add_mounted_fs (UDisksCleanup  *cleanup,
       g_variant_iter_init (&iter, value);
       while ((child = g_variant_iter_next_value (&iter)) != NULL)
         {
-          g_variant_builder_add_value (&builder, child);
+          const gchar *entry_mount_point;
+          g_variant_get (child, "{&s@a{sv}}", &entry_mount_point, NULL);
+          /* Skip/remove stale entries */
+          if (g_strcmp0 (entry_mount_point, mount_point) == 0)
+            {
+              udisks_warning ("Removing stale entry for mount point `%s' in /run/udisks2/mounted-fs file",
+                              entry_mount_point);
+            }
+          else
+            {
+              g_variant_builder_add_value (&builder, child);
+            }
           g_variant_unref (child);
         }
       g_variant_unref (value);
@@ -925,7 +960,7 @@ udisks_cleanup_add_mounted_fs (UDisksCleanup  *cleanup,
  * @out_fstab_mount: Return location for whether the device was a fstab mount or %NULL.
  *
  * Gets the mount point for @block_device, if it exists in the
- * <filename>/var/lib/udisks2/mounted-fs</filename> file.
+ * <filename>/run/udisks2/mounted-fs</filename> file.
  *
  * Returns: The mount point for @block_device or %NULL if not found.
  */
@@ -988,24 +1023,24 @@ udisks_cleanup_find_mounted_fs (UDisksCleanup   *cleanup,
                   ret = g_strdup (mount_point);
                   if (out_uid != NULL)
                     {
-                      GVariant *value;
-                      value = lookup_asv (details, "mounted-by-uid");
+                      GVariant *lookup_value;
+                      lookup_value = lookup_asv (details, "mounted-by-uid");
                       *out_uid = 0;
-                      if (value != NULL)
+                      if (lookup_value != NULL)
                         {
-                          *out_uid = g_variant_get_uint32 (value);
-                          g_variant_unref (value);
+                          *out_uid = g_variant_get_uint32 (lookup_value);
+                          g_variant_unref (lookup_value);
                         }
                     }
                   if (out_fstab_mount != NULL)
                     {
-                      GVariant *value;
-                      value = lookup_asv (details, "fstab-mount");
+                      GVariant *lookup_value;
+                      lookup_value = lookup_asv (details, "fstab-mount");
                       *out_fstab_mount = FALSE;
-                      if (value != NULL)
+                      if (lookup_value != NULL)
                         {
-                          *out_fstab_mount = g_variant_get_boolean (value);
-                          g_variant_unref (value);
+                          *out_fstab_mount = g_variant_get_boolean (lookup_value);
+                          g_variant_unref (lookup_value);
                         }
                     }
                   g_variant_unref (block_device_value);
@@ -1157,6 +1192,7 @@ udisks_cleanup_check_unlocked_luks_entry (UDisksCleanup  *cleanup,
           escaped_device_file = udisks_daemon_util_escape_and_quote (device_file_cleartext);
           if (!udisks_daemon_launch_spawned_job_sync (cleanup->daemon,
                                                       NULL, /* UDisksObject */
+                                                      "cleanup", 0, /* StartedByUID */
                                                       NULL, /* GCancellable */
                                                       0,    /* uid_t run_as_uid */
                                                       0,    /* uid_t run_as_euid */
@@ -1328,7 +1364,19 @@ udisks_cleanup_add_unlocked_luks (UDisksCleanup  *cleanup,
       g_variant_iter_init (&iter, value);
       while ((child = g_variant_iter_next_value (&iter)) != NULL)
         {
-          g_variant_builder_add_value (&builder, child);
+          guint64 entry_cleartext_device;
+          g_variant_get (child, "{t@a{sv}}", &entry_cleartext_device, NULL);
+          /* Skip/remove stale entries */
+          if ((dev_t) entry_cleartext_device == cleartext_device)
+            {
+              udisks_warning ("Removing stale entry for cleartext device %d:%d in /run/udisks2/unlocked-luks file",
+                              (gint) major (entry_cleartext_device),
+                              (gint) minor (entry_cleartext_device));
+            }
+          else
+            {
+              g_variant_builder_add_value (&builder, child);
+            }
           g_variant_unref (child);
         }
       g_variant_unref (value);
@@ -1445,13 +1493,13 @@ udisks_cleanup_find_unlocked_luks (UDisksCleanup   *cleanup,
                   ret = cleartext_device;
                   if (out_uid != NULL)
                     {
-                      GVariant *value;
-                      value = lookup_asv (details, "unlocked-by-uid");
+                      GVariant *lookup_value;
+                      lookup_value = lookup_asv (details, "unlocked-by-uid");
                       *out_uid = 0;
-                      if (value != NULL)
+                      if (lookup_value != NULL)
                         {
-                          *out_uid = g_variant_get_uint32 (value);
-                          g_variant_unref (value);
+                          *out_uid = g_variant_get_uint32 (lookup_value);
+                          g_variant_unref (lookup_value);
                         }
                     }
                   g_variant_unref (crypto_device_value);
@@ -1483,34 +1531,14 @@ udisks_cleanup_check_loop_entry (UDisksCleanup  *cleanup,
                                  GArray         *devs_to_clean)
 {
   const gchar *loop_device;
-  GVariant *details;
-  gchar *s;
-  gboolean keep;
-  gboolean is_setup;
-  gboolean has_backing_device;
-  gboolean backing_device_mounted;
-  gboolean attempt_no_cleanup;
-  GVariant *backing_file_value;
-  GVariant *backing_file_device_value;
+  GVariant *details = NULL;
+  gboolean keep = FALSE;
+  GVariant *backing_file_value = NULL;
   const gchar *backing_file;
-  dev_t backing_file_device;
   GUdevClient *udev_client;
-  struct stat loop_device_statbuf;
-  gint loop_device_fd;
-  struct loop_info64 li64;
-  UDisksMountMonitor *monitor;
-  GUdevDevice *udev_backing_file_device;
+  GUdevDevice *device = NULL;
+  const gchar *sysfs_backing_file;
 
-  keep = FALSE;
-  attempt_no_cleanup = FALSE;
-  is_setup = FALSE;
-  has_backing_device = FALSE;
-  backing_device_mounted = FALSE;
-  backing_file_value = NULL;
-  backing_file_device_value = NULL;
-  details = NULL;
-
-  monitor = udisks_daemon_get_mount_monitor (cleanup->daemon);
   udev_client = udisks_linux_provider_get_udev_client (udisks_daemon_get_linux_provider (cleanup->daemon));
 
   g_variant_get (value,
@@ -1521,155 +1549,68 @@ udisks_cleanup_check_loop_entry (UDisksCleanup  *cleanup,
   backing_file_value = lookup_asv (details, "backing-file");
   if (backing_file_value == NULL)
     {
+      gchar *s;
       s = g_variant_print (value, TRUE);
       udisks_error ("loop entry %s is invalid: no backing-file key/value pair", s);
       g_free (s);
-      attempt_no_cleanup = TRUE;
       goto out;
     }
   backing_file = g_variant_get_bytestring (backing_file_value);
 
-  backing_file_device_value = lookup_asv (details, "backing-file-device");
-  if (backing_file_device_value == NULL)
+  /* check the loop device is still set up */
+  device = g_udev_client_query_by_device_file (udev_client, loop_device);
+  if (device == NULL)
     {
-      s = g_variant_print (value, TRUE);
-      udisks_error ("loop entry %s is invalid: no backing-file-device key/value pair", s);
-      g_free (s);
-      attempt_no_cleanup = TRUE;
+      udisks_info ("no udev device for %s", loop_device);
       goto out;
     }
-  backing_file_device = g_variant_get_uint64 (backing_file_device_value);
-
-  if (backing_file_device == 0 || major (backing_file_device) == 0)
+  if (g_udev_device_get_sysfs_attr (device, "loop/offset") == NULL)
     {
-      /* major==0 -> not regular block device ... could be e.g. NFS ...
-       * for now, just assume it's still there and mounted
-       */
-      keep = TRUE;
+      udisks_info ("loop device %s is not setup  (no loop/offset sysfs file)", loop_device);
       goto out;
     }
 
-  if (stat (loop_device, &loop_device_statbuf) != 0)
+  /* Check the loop device set up, is the one that _we_ set up
+   *
+   * Note that drivers/block/loop.c:loop_attr_backing_file_show() uses d_path()
+   * on lo_file_name so in the event that the underlying fs was unmounted
+   * (just 'umount -l /path/to/fs/holding/backing/file to try) it cuts
+   * off the mount path.... in this case we simply just give up managing
+   * the loop device
+   */
+  sysfs_backing_file = g_udev_device_get_sysfs_attr (device, "loop/backing_file");
+  if (g_strcmp0 (sysfs_backing_file, backing_file) != 0)
     {
-      udisks_error ("error statting %s: %m", loop_device);
-      attempt_no_cleanup = TRUE;
+      udisks_notice ("unexpected name for %s - expected `%s' but got `%s'",
+                     loop_device, backing_file, sysfs_backing_file);
       goto out;
     }
-  loop_device_fd = open (loop_device, O_RDONLY);
-  if (loop_device_fd == -1 )
-    {
-      udisks_info ("error opening %s: %m", loop_device);
-      attempt_no_cleanup = TRUE;
-      goto out;
-    }
-  memset (&li64, '\0', sizeof (struct loop_info64));
-  if (ioctl (loop_device_fd, LOOP_GET_STATUS64, &li64) == -1)
-    {
-      udisks_info ("error issuing LOOP_GET_STATUS64 ioctl on %s: %m", loop_device);
-      attempt_no_cleanup = TRUE;
-      close (loop_device_fd);
-      goto out;
-    }
-  close (loop_device_fd);
-  if (strncmp ((const char *) li64.lo_file_name, backing_file, LO_NAME_SIZE - 1) != 0)
-    {
-      udisks_error ("unexpected name for device %s - expected `%s' but got `%s'",
-                    loop_device, backing_file, li64.lo_file_name);
-      attempt_no_cleanup = TRUE;
-      goto out;
-    }
-  is_setup = TRUE;
-
-  /* check with udev if the backing device exists and is still mounted */
-  udev_backing_file_device = g_udev_client_query_by_device_number (udev_client,
-                                                                   G_UDEV_DEVICE_TYPE_BLOCK,
-                                                                   backing_file_device);
-  if (udev_backing_file_device != NULL)
-    {
-      GList *mounts;
-
-      has_backing_device = TRUE;
-      g_object_unref (udev_backing_file_device);
-
-      /* and if it's still mounted */
-      mounts = udisks_mount_monitor_get_mounts_for_dev (monitor, backing_file_device);
-      if (mounts != NULL)
-        backing_device_mounted = TRUE;
-      g_list_foreach (mounts, (GFunc) g_object_unref, NULL);
-      g_list_free (mounts);
-    }
 
   /* OK, entry is valid - keep it around */
-  if (is_setup && has_backing_device && backing_device_mounted)
-    keep = TRUE;
+  keep = TRUE;
 
  out:
 
   if (check_only && !keep)
     {
-      g_array_append_val (devs_to_clean, loop_device_statbuf.st_rdev);
+      if (device != NULL)
+        {
+          dev_t dev_number = g_udev_device_get_device_number (device);
+          g_array_append_val (devs_to_clean, dev_number);
+        }
       keep = TRUE;
       goto out2;
     }
 
   if (!keep)
     {
-      if (!attempt_no_cleanup)
-        {
-          if (is_setup)
-            {
-              gchar *escaped_loop_device_file;
-              gchar *error_message;
-
-              if (!has_backing_device)
-                udisks_notice ("Cleaning up loop device %s (backing device %d:%d no longer exist)",
-                               loop_device,
-                               major (backing_file_device), minor (backing_file_device));
-              else
-                udisks_notice ("Cleaning up loop device %s (backing device %d:%d no longer mounted)",
-                               loop_device,
-                               major (backing_file_device), minor (backing_file_device));
-
-              error_message = NULL;
-              escaped_loop_device_file = udisks_daemon_util_escape_and_quote (loop_device);
-              if (!udisks_daemon_launch_spawned_job_sync (cleanup->daemon,
-                                                          NULL, /* UDisksObject */
-                                                          NULL, /* GCancellable */
-                                                          0,    /* uid_t run_as_uid */
-                                                          0,    /* uid_t run_as_euid */
-                                                          NULL, /* gint *out_status */
-                                                          &error_message,
-                                                          NULL,  /* input_string */
-                                                          "losetup -d %s",
-                                                          escaped_loop_device_file))
-                {
-                  udisks_error ("Error cleaning up loop device %s: %s",
-                                loop_device, error_message);
-                  g_free (escaped_loop_device_file);
-                  g_free (error_message);
-                  /* keep the entry so we can clean it up later */
-                  keep = TRUE;
-                  goto out2;
-                }
-              g_free (escaped_loop_device_file);
-              g_free (error_message);
-            }
-          else
-            {
-              udisks_notice ("loop device %s was manually deleted", loop_device);
-            }
-        }
-      else
-        {
-          udisks_notice ("Loop device %s was deleted", loop_device);
-        }
+      udisks_notice ("No longer watching loop device %s", loop_device);
     }
 
  out2:
+  g_clear_object (&device);
   if (backing_file_value != NULL)
     g_variant_unref (backing_file_value);
-  if (backing_file_device_value != NULL)
-    g_variant_unref (backing_file_device_value);
   if (details != NULL)
     g_variant_unref (details);
   return keep;
@@ -1808,7 +1749,18 @@ udisks_cleanup_add_loop (UDisksCleanup   *cleanup,
       g_variant_iter_init (&iter, value);
       while ((child = g_variant_iter_next_value (&iter)) != NULL)
         {
-          g_variant_builder_add_value (&builder, child);
+          const gchar *entry_loop_device;
+          g_variant_get (child, "{&s@a{sv}}", &entry_loop_device, NULL);
+          /* Skip/remove stale entries */
+          if (g_strcmp0 (entry_loop_device, device_file) == 0)
+            {
+              udisks_warning ("Removing stale entry for loop device `%s' in /run/udisks2/loop file",
+                              entry_loop_device);
+            }
+          else
+            {
+              g_variant_builder_add_value (&builder, child);
+            }
           g_variant_unref (child);
         }
       g_variant_unref (value);
@@ -1919,13 +1871,13 @@ udisks_cleanup_has_loop (UDisksCleanup   *cleanup,
               ret = TRUE;
               if (out_uid != NULL)
                 {
-                  GVariant *value;
-                  value = lookup_asv (details, "setup-by-uid");
+                  GVariant *lookup_value;
+                  lookup_value = lookup_asv (details, "setup-by-uid");
                   *out_uid = 0;
-                  if (value != NULL)
+                  if (lookup_value != NULL)
                     {
-                      *out_uid = g_variant_get_uint32 (value);
-                      g_variant_unref (value);
+                      *out_uid = g_variant_get_uint32 (lookup_value);
+                      g_variant_unref (lookup_value);
                     }
                 }
               g_variant_unref (details);
@@ -1944,3 +1896,336 @@ udisks_cleanup_has_loop (UDisksCleanup   *cleanup,
   return ret;
 }
 
+/* ---------------------------------------------------------------------------------------------------- */
+
+/* returns TRUE if the entry should be kept */
+static gboolean
+udisks_cleanup_check_mdraid_entry (UDisksCleanup  *cleanup,
+                                   GVariant       *value,
+                                   gboolean        check_only,
+                                   GArray         *devs_to_clean)
+{
+  dev_t raid_device;
+  GVariant *details = NULL;
+  gboolean keep = FALSE;
+  GUdevClient *udev_client;
+  GUdevDevice *device = NULL;
+  const gchar *array_state;
+
+  udev_client = udisks_linux_provider_get_udev_client (udisks_daemon_get_linux_provider (cleanup->daemon));
+
+  g_variant_get (value,
+                 "{t@a{sv}}",
+                 &raid_device,
+                 &details);
+
+  /* check if the RAID device is still set up */
+  device = g_udev_client_query_by_device_number (udev_client, G_UDEV_DEVICE_TYPE_BLOCK, raid_device);
+  if (device == NULL)
+    {
+      udisks_info ("no udev device for raid device %d:%d", major (raid_device), minor (raid_device));
+      goto out;
+    }
+  array_state = g_udev_device_get_sysfs_attr (device, "md/array_state");
+  if (array_state == NULL)
+    {
+      udisks_info ("raid device %d:%d is not setup  (no md/array_state sysfs file)",
+                   major (raid_device), minor (raid_device));
+      goto out;
+    }
+
+  if (g_strcmp0 (array_state, "clear") == 0)
+    {
+      /* 'clear' means that the array is not set up any more */
+      goto out;
+    }
+
+  /* OK, entry is valid - keep it around */
+  keep = TRUE;
+
+ out:
+
+  if (check_only && !keep)
+    {
+      if (device != NULL)
+        {
+          g_array_append_val (devs_to_clean, raid_device);
+        }
+      keep = TRUE;
+      goto out2;
+    }
+
+  if (!keep)
+    {
+      udisks_notice ("No longer watching mdraid device %d:%d", major (raid_device), minor (raid_device));
+    }
+
+ out2:
+  g_clear_object (&device);
+  if (details != NULL)
+    g_variant_unref (details);
+  return keep;
+}
+
+static void
+udisks_cleanup_check_mdraid (UDisksCleanup *cleanup,
+                             gboolean       check_only,
+                             GArray        *devs_to_clean)
+{
+  gboolean changed;
+  GVariant *value;
+  GVariant *new_value;
+  GVariantBuilder builder;
+  GError *error;
+
+  changed = FALSE;
+
+  /* load existing entries */
+  error = NULL;
+  value = udisks_persistent_store_get (cleanup->persistent_store,
+                                       UDISKS_PERSISTENT_FLAGS_TEMPORARY_STORE,
+                                       "mdraid",
+                                       G_VARIANT_TYPE ("a{ta{sv}}"),
+                                       &error);
+  if (error != NULL)
+    {
+      udisks_warning ("Error getting mdraid: %s (%s, %d)",
+                      error->message, g_quark_to_string (error->domain), error->code);
+      g_error_free (error);
+      goto out;
+    }
+
+  /* check valid entries */
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ta{sv}}"));
+  if (value != NULL)
+    {
+      GVariantIter iter;
+      GVariant *child;
+      g_variant_iter_init (&iter, value);
+      while ((child = g_variant_iter_next_value (&iter)) != NULL)
+        {
+          if (udisks_cleanup_check_mdraid_entry (cleanup, child, check_only, devs_to_clean))
+            g_variant_builder_add_value (&builder, child);
+          else
+            changed = TRUE;
+          g_variant_unref (child);
+        }
+      g_variant_unref (value);
+    }
+
+  new_value = g_variant_builder_end (&builder);
+
+  /* save new entries */
+  if (changed)
+    {
+      error = NULL;
+      if (!udisks_persistent_store_set (cleanup->persistent_store,
+                                        UDISKS_PERSISTENT_FLAGS_TEMPORARY_STORE,
+                                        "mdraid",
+                                        G_VARIANT_TYPE ("a{ta{sv}}"),
+                                        new_value, /* consumes new_value */
+                                        &error))
+        {
+          udisks_warning ("Error setting mdraid: %s (%s, %d)",
+                          error->message,
+                          g_quark_to_string (error->domain),
+                          error->code);
+          g_error_free (error);
+          goto out;
+        }
+    }
+  else
+    {
+      g_variant_unref (new_value);
+    }
+
+ out:
+  ;
+}
+
+/**
+ * udisks_cleanup_add_mdraid:
+ * @cleanup: A #UDisksCleanup.
+ * @raid_device: The #dev_t for the RAID device.
+ * @uid: The user id of the process requesting the loop device.
+ *
+ * Adds a new entry to the <filename>/run/udisks2/mdraid</filename>
+ * file.
+ */
+void
+udisks_cleanup_add_mdraid (UDisksCleanup   *cleanup,
+                           dev_t            raid_device,
+                           uid_t            uid)
+{
+  GVariant *value;
+  GVariant *new_value;
+  GVariant *details_value;
+  GVariantBuilder builder;
+  GVariantBuilder details_builder;
+  GError *error;
+
+  g_return_if_fail (UDISKS_IS_CLEANUP (cleanup));
+
+  g_mutex_lock (&cleanup->lock);
+
+  /* load existing entries */
+  error = NULL;
+  value = udisks_persistent_store_get (cleanup->persistent_store,
+                                       UDISKS_PERSISTENT_FLAGS_TEMPORARY_STORE,
+                                       "mdraid",
+                                       G_VARIANT_TYPE ("a{ta{sv}}"),
+                                       &error);
+  if (error != NULL)
+    {
+      udisks_warning ("Error getting mdraid: %s (%s, %d)",
+                      error->message, g_quark_to_string (error->domain), error->code);
+      g_error_free (error);
+      goto out;
+    }
+
+  /* start by including existing entries */
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ta{sv}}"));
+  if (value != NULL)
+    {
+      GVariantIter iter;
+      GVariant *child;
+      g_variant_iter_init (&iter, value);
+      while ((child = g_variant_iter_next_value (&iter)) != NULL)
+        {
+          guint64 entry_raid_device;
+          g_variant_get (child, "{t@a{sv}}", &entry_raid_device, NULL);
+          /* Skip/remove stale entries */
+          if (entry_raid_device == raid_device)
+            {
+              udisks_warning ("Removing stale entry for raid device %d:%d in /run/udisks2/mdraid file",
+                              major (raid_device), minor (raid_device));
+            }
+          else
+            {
+              g_variant_builder_add_value (&builder, child);
+            }
+          g_variant_unref (child);
+        }
+      g_variant_unref (value);
+    }
+
+  /* build the details */
+  g_variant_builder_init (&details_builder, G_VARIANT_TYPE ("a{sv}"));
+  g_variant_builder_add (&details_builder,
+                         "{sv}",
+                         "started-by-uid",
+                         g_variant_new_uint32 (uid));
+  details_value = g_variant_builder_end (&details_builder);
+
+  /* finally add the new entry */
+  g_variant_builder_add (&builder,
+                         "{t@a{sv}}",
+                         raid_device,
+                         details_value); /* consumes details_value */
+  new_value = g_variant_builder_end (&builder);
+
+  /* save new entries */
+  error = NULL;
+  if (!udisks_persistent_store_set (cleanup->persistent_store,
+                                    UDISKS_PERSISTENT_FLAGS_TEMPORARY_STORE,
+                                    "mdraid",
+                                    G_VARIANT_TYPE ("a{ta{sv}}"),
+                                    new_value, /* consumes new_value */
+                                    &error))
+    {
+      udisks_warning ("Error setting mdraid: %s (%s, %d)",
+                      error->message, g_quark_to_string (error->domain), error->code);
+      g_error_free (error);
+      goto out;
+    }
+
+ out:
+  g_mutex_unlock (&cleanup->lock);
+}
+
+/**
+ * udisks_cleanup_has_mdraid:
+ * @cleanup: A #UDisksCleanup
+ * @raid_device: A #dev_t for the RAID device.
+ * @out_uid: Return location for the user id who setup the loop device or %NULL.
+ * @error: Return location for error or %NULL.
+ *
+ * Checks if @raid_device is set up via udisks.
+ *
+ * Returns: %TRUE if set up via udisks, otherwise %FALSE or if @error is set.
+ */
+gboolean
+udisks_cleanup_has_mdraid (UDisksCleanup   *cleanup,
+                           dev_t            raid_device,
+                           uid_t           *out_uid)
+{
+  gboolean ret = FALSE;
+  GVariant *value = NULL;
+  GError *error = NULL;
+
+  g_return_val_if_fail (UDISKS_IS_CLEANUP (cleanup), FALSE);
+
+  g_mutex_lock (&cleanup->lock);
+
+  /* load existing entries */
+  value = udisks_persistent_store_get (cleanup->persistent_store,
+                                       UDISKS_PERSISTENT_FLAGS_TEMPORARY_STORE,
+                                       "mdraid",
+                                       G_VARIANT_TYPE ("a{ta{sv}}"),
+                                       &error);
+  if (error != NULL)
+    {
+      udisks_warning ("Error getting mdraid: %s (%s, %d)",
+                      error->message, g_quark_to_string (error->domain), error->code);
+      g_clear_error (&error);
+      goto out;
+    }
+
+  /* look through list */
+  if (value != NULL)
+    {
+      GVariantIter iter;
+      GVariant *child;
+      g_variant_iter_init (&iter, value);
+      while ((child = g_variant_iter_next_value (&iter)) != NULL)
+        {
+          guint64 iter_raid_device;
+          GVariant *details;
+
+          g_variant_get (child,
+                         "{t@a{sv}}",
+                         &iter_raid_device,
+                         &details);
+
+          if (iter_raid_device == raid_device)
+            {
+              ret = TRUE;
+              if (out_uid != NULL)
+                {
+                  GVariant *lookup_value;
+                  lookup_value = lookup_asv (details, "started-by-uid");
+                  *out_uid = 0;
+                  if (lookup_value != NULL)
+                    {
+                      *out_uid = g_variant_get_uint32 (lookup_value);
+                      g_variant_unref (lookup_value);
+                    }
+                }
+              g_variant_unref (details);
+              g_variant_unref (child);
+              goto out;
+            }
+          g_variant_unref (details);
+          g_variant_unref (child);
+        }
+    }
+
+ out:
+  if (value != NULL)
+    g_variant_unref (value);
+  g_mutex_unlock (&cleanup->lock);
+  return ret;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
