@@ -38,6 +38,7 @@
 #include "udiskscrypttabmonitor.h"
 #include "udiskscrypttabentry.h"
 #include "udiskslinuxblockobject.h"
+#include "udiskslinuxdevice.h"
 
 /**
  * SECTION:udisksdaemon
@@ -500,6 +501,8 @@ static guint job_id = 0;
  * udisks_daemon_launch_simple_job:
  * @daemon: A #UDisksDaemon.
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
+ * @job_operation: The operation for the job.
+ * @job_started_by_uid: The user who started the job.
  * @cancellable: A #GCancellable or %NULL.
  *
  * Launches a new simple job.
@@ -558,6 +561,8 @@ udisks_daemon_launch_simple_job (UDisksDaemon    *daemon,
  * udisks_daemon_launch_threaded_job:
  * @daemon: A #UDisksDaemon.
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
+ * @job_operation: The operation for the job.
+ * @job_started_by_uid: The user who started the job.
  * @job_func: The function to run in another thread.
  * @user_data: User data to pass to @job_func.
  * @user_data_free_func: Function to free @user_data with or %NULL.
@@ -629,6 +634,8 @@ udisks_daemon_launch_threaded_job  (UDisksDaemon    *daemon,
  * udisks_daemon_launch_spawned_job:
  * @daemon: A #UDisksDaemon.
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
+ * @job_operation: The operation for the job.
+ * @job_started_by_uid: The user who started the job.
  * @cancellable: A #GCancellable or %NULL.
  * @run_as_uid: The #uid_t to run the command as.
  * @run_as_euid: The effective #uid_t to run the command as.
@@ -739,6 +746,8 @@ spawned_job_sync_on_completed (UDisksJob    *job,
  * udisks_daemon_launch_spawned_job_sync:
  * @daemon: A #UDisksDaemon.
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
+ * @job_operation: The operation for the job.
+ * @job_started_by_uid: The user who started the job.
  * @cancellable: A #GCancellable or %NULL.
  * @run_as_uid: The #uid_t to run the command as.
  * @run_as_euid: The effective #uid_t to run the command as.
@@ -1051,7 +1060,7 @@ udisks_daemon_find_block_by_sysfs_path (UDisksDaemon *daemon,
   for (l = objects; l != NULL; l = l->next)
     {
       UDisksObject *object = UDISKS_OBJECT (l->data);
-      GUdevDevice *device;
+      UDisksLinuxDevice *device;
 
       if (!UDISKS_IS_LINUX_BLOCK_OBJECT (object))
         continue;
@@ -1060,7 +1069,7 @@ udisks_daemon_find_block_by_sysfs_path (UDisksDaemon *daemon,
       if (device == NULL)
         continue;
 
-      if (g_strcmp0 (g_udev_device_get_sysfs_path (device), sysfs_path) == 0)
+      if (g_strcmp0 (g_udev_device_get_sysfs_path (device->udev_device), sysfs_path) == 0)
         {
           g_object_unref (device);
           ret = g_object_ref (object);
