@@ -42,6 +42,7 @@
 #include "udiskslinuxdriveobject.h"
 
 #if defined(HAVE_LIBSYSTEMD_LOGIN)
+#include <systemd/sd-daemon.h>
 #include <systemd/sd-login.h>
 #endif
 
@@ -257,10 +258,13 @@ udisks_daemon_util_resolve_link (const gchar *path,
   if (num != -1)
     {
       char *absolute_path;
+      gchar *full_path_dir;
 
       link_path[num] = '\0';
 
-      absolute_path = g_build_filename (path, link_path, NULL);
+      full_path_dir = g_path_get_dirname (full_path);
+      absolute_path = g_build_filename (full_path_dir, link_path, NULL);
+      g_free (full_path_dir);
       if (realpath (absolute_path, resolved_path) != NULL)
         {
           found_it = TRUE;
@@ -396,6 +400,20 @@ udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
       g_object_unref (crypto_object);
     }
 
+  /* MDRaid devices */
+  if (g_strcmp0 (udisks_block_get_mdraid (block), "/") != 0)
+    {
+      uid_t started_by_user;
+      if (udisks_cleanup_has_mdraid (cleanup, udisks_block_get_device_number (block), &started_by_user))
+        {
+          if (started_by_user == user)
+            {
+              ret = TRUE;
+              goto out;
+            }
+        }
+    }
+
  out:
   g_clear_object (&partition);
   g_clear_object (&block);
@@ -993,7 +1011,7 @@ udisks_daemon_util_escape (const gchar *str)
 }
 
 /**
- * udisks_daemon_util_on_other_seat:
+ * udisks_daemon_util_on_same_seat:
  * @daemon: A #UDisksDaemon.
  * @object: The #GDBusObject that the call is on or %NULL.
  * @process: The process to check for.
@@ -1021,6 +1039,10 @@ udisks_daemon_util_on_same_seat (UDisksDaemon          *daemon,
   UDisksObject *drive_object = NULL;
   UDisksDrive *drive = NULL;
 
+  /* if we haven't booted with systemd, assume it's always the same seat */
+  if (sd_booted () <= 0)
+    return TRUE;
+
   if (UDISKS_IS_LINUX_BLOCK_OBJECT (object))
     {
       UDisksLinuxBlockObject *linux_block_object;
@@ -1132,9 +1154,10 @@ udisks_daemon_util_hexdump_debug (gconstpointer data, gsize len)
 /* ---------------------------------------------------------------------------------------------------- */
 
 /**
+ * udisks_daemon_util_file_set_contents:
  * @filename: (type filename): Name of a file to write @contents to, in the GLib file name encoding.
  * @contents: (array length=length) (element-type guint8): String to write to the file.
- * @length: Length of @contents, or -1 if @contents is a NUL-terminated string.
+ * @contents_len: Length of @contents, or -1 if @contents is a NUL-terminated string.
  * @mode_for_new_file: Mode for new file.
  * @error: Return location for a #GError, or %NULL.
  *
@@ -1375,3 +1398,64 @@ udisks_daemon_util_uninhibit_system_sync (UDisksInhibitCookie *cookie)
 #endif
 }
 
+/**
+ * udisks_daemon_util_get_free_mdraid_device:
+ *
+ * Gets a free MD RAID device.
+ *
+ * Returns: A string of the form "/dev/mdNNN" that should be freed
+ * with g_free() or %NULL if no free device is available.
+ */
+gchar *
+udisks_daemon_util_get_free_mdraid_device (void)
+{
+  gchar *ret = NULL;
+  gint n;
+  gchar buf[PATH_MAX];
+
+  /* Ideally we wouldn't need this racy function... but mdadm(8)
+   * insists that the user chooses a name. It should just choose one
+   * itself but that's not how things work right now.
+   */
+  for (n = 127; n >= 0; n--)
+    {
+      snprintf (buf, sizeof buf, "/sys/block/md%d", n);
+      if (!g_file_test (buf, G_FILE_TEST_EXISTS))
+        {
+          ret = g_strdup_printf ("/dev/md%d", n);
+          goto out;
+        }
+    }
+
+ out:
+  return ret;
+}
+
+
+/**
+ * udisks_ata_identify_get_word:
+ * @identify_data: (allow-none): A 512-byte array containing ATA IDENTIFY or ATA IDENTIFY PACKET DEVICE data or %NULL.
+ * @word_number: The word number to get - must be less than 256.
+ *
+ * Gets a <quote>word</quote> from position @word_number from
+ * @identify_data.
+ *
+ * Returns: The word at the specified position or 0 if @identify_data is %NULL.
+ */
+guint16
+udisks_ata_identify_get_word (const guchar *identify_data, guint word_number)
+{
+  const guint16 *words = (const guint16 *) identify_data;
+  guint16 ret = 0;
+
+  g_return_val_if_fail (word_number < 256, 0);
+
+  if (identify_data == NULL)
+    goto out;
+
+  ret = GUINT16_FROM_LE (words[word_number]);
+
+ out:
+  return ret;
+}
+
