@@ -33,6 +33,7 @@
 #include "udiskslinuxdrive.h"
 #include "udiskslinuxdriveata.h"
 #include "udiskslinuxblockobject.h"
+#include "udiskslinuxdevice.h"
 
 /**
  * SECTION:udiskslinuxdriveobject
@@ -56,7 +57,7 @@ struct _UDisksLinuxDriveObject
 
   UDisksDaemon *daemon;
 
-  /* list of GUdevDevice objects for block objects */
+  /* list of UDisksLinuxDevice objects for block objects */
   GList *devices;
 
   /* interfaces */
@@ -64,8 +65,6 @@ struct _UDisksLinuxDriveObject
   UDisksDriveAta *iface_drive_ata;
 };
 
-// G_LOCK_DEFINE_STATIC (drive_object_lock);
-
 struct _UDisksLinuxDriveObjectClass
 {
   UDisksObjectSkeletonClass parent_class;
@@ -99,12 +98,12 @@ udisks_linux_drive_object_finalize (GObject *_object)
 }
 
 static void
-udisks_linux_drive_object_get_property (GObject    *_object,
+udisks_linux_drive_object_get_property (GObject    *__object,
                                         guint       prop_id,
                                         GValue     *value,
                                         GParamSpec *pspec)
 {
-  UDisksLinuxDriveObject *object = UDISKS_LINUX_DRIVE_OBJECT (_object);
+  UDisksLinuxDriveObject *object = UDISKS_LINUX_DRIVE_OBJECT (__object);
 
   switch (prop_id)
     {
@@ -113,18 +112,18 @@ udisks_linux_drive_object_get_property (GObject    *_object,
       break;
 
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (_object, prop_id, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
 static void
-udisks_linux_drive_object_set_property (GObject      *_object,
+udisks_linux_drive_object_set_property (GObject      *__object,
                                         guint         prop_id,
                                         const GValue *value,
                                         GParamSpec   *pspec)
 {
-  UDisksLinuxDriveObject *object = UDISKS_LINUX_DRIVE_OBJECT (_object);
+  UDisksLinuxDriveObject *object = UDISKS_LINUX_DRIVE_OBJECT (__object);
 
   switch (prop_id)
     {
@@ -140,7 +139,7 @@ udisks_linux_drive_object_set_property (GObject      *_object,
       break;
 
     default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (_object, prop_id, pspec);
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
@@ -172,7 +171,7 @@ udisks_linux_drive_object_constructor (GType                  type,
   GObjectConstructParam *cp;
   UDisksDaemon *daemon;
   GUdevClient *client;
-  GUdevDevice *device;
+  UDisksLinuxDevice *device;
 
   cp = find_construct_property (n_construct_properties, construct_properties, "daemon");
   g_assert (cp != NULL);
@@ -183,7 +182,7 @@ udisks_linux_drive_object_constructor (GType                  type,
 
   cp = find_construct_property (n_construct_properties, construct_properties, "device");
   g_assert (cp != NULL);
-  device = G_UDEV_DEVICE (g_value_get_object (cp->value));
+  device = g_value_get_object (cp->value);
   g_assert (device != NULL);
 
   if (!udisks_linux_drive_object_should_include_device (client, device, NULL))
@@ -303,7 +302,7 @@ udisks_linux_drive_object_class_init (UDisksLinuxDriveObjectClass *klass)
   /**
    * UDisksLinuxDriveObject:device:
    *
-   * The #GUdevDevice for the object. Connect to the #GObject::notify
+   * The #UDisksLinuxDevice for the object. Connect to the #GObject::notify
    * signal to get notified whenever this is updated.
    */
   g_object_class_install_property (gobject_class,
@@ -311,7 +310,7 @@ udisks_linux_drive_object_class_init (UDisksLinuxDriveObjectClass *klass)
                                    g_param_spec_object ("device",
                                                         "Device",
                                                         "The device for the object",
-                                                        G_UDEV_TYPE_DEVICE,
+                                                        UDISKS_TYPE_LINUX_DEVICE,
                                                         G_PARAM_WRITABLE |
                                                         G_PARAM_CONSTRUCT_ONLY |
                                                         G_PARAM_STATIC_STRINGS));
@@ -321,20 +320,20 @@ udisks_linux_drive_object_class_init (UDisksLinuxDriveObjectClass *klass)
 /**
  * udisks_linux_drive_object_new:
  * @daemon: A #UDisksDaemon.
- * @device: The #GUdevDevice for the sysfs block device.
+ * @device: The #UDisksLinuxDevice for the sysfs block device.
  *
  * Create a new drive object.
  *
  * Returns: A #UDisksLinuxDriveObject object or %NULL if @device does not represent a drive. Free with g_object_unref().
  */
 UDisksLinuxDriveObject *
-udisks_linux_drive_object_new (UDisksDaemon  *daemon,
-                               GUdevDevice   *device)
+udisks_linux_drive_object_new (UDisksDaemon      *daemon,
+                               UDisksLinuxDevice *device)
 {
   GObject *object;
 
   g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
-  g_return_val_if_fail (G_UDEV_IS_DEVICE (device), NULL);
+  g_return_val_if_fail (UDISKS_IS_LINUX_DEVICE (device), NULL);
 
   object = g_object_new (UDISKS_TYPE_LINUX_DRIVE_OBJECT,
                          "daemon", daemon,
@@ -366,9 +365,9 @@ udisks_linux_drive_object_get_daemon (UDisksLinuxDriveObject *object)
  * udisks_linux_drive_object_get_devices:
  * @object: A #UDisksLinuxDriveObject.
  *
- * Gets the current #GUdevDevice objects associated with @object.
+ * Gets the current #UDisksLinuxDevice objects associated with @object.
  *
- * Returns: A list of #GUdevDevice objects. Free each element with
+ * Returns: A list of #UDisksLinuxDevice objects. Free each element with
  * g_object_unref(), then free the list with g_list_free().
  */
 GList *
@@ -386,20 +385,20 @@ udisks_linux_drive_object_get_devices (UDisksLinuxDriveObject *object)
  * @object: A #UDisksLinuxDriveObject.
  * @get_hw: If the drive is multipath, set to %TRUE to get a path device instead of the multipath device.
  *
- * Gets one of the #GUdevDevice object associated with @object.
+ * Gets one of the #UDisksLinuxDevice object associated with @object.
  *
  * If @get_hw is %TRUE and @object represents a multipath device then
  * one of the paths is returned rather than the multipath device. This
  * is useful if you e.g. need to configure the physical hardware.
  *
- * Returns: A #GUdevDevice or %NULL. The returned object must be freed
+ * Returns: A #UDisksLinuxDevice or %NULL. The returned object must be freed
  * with g_object_unref().
  */
-GUdevDevice *
+UDisksLinuxDevice *
 udisks_linux_drive_object_get_device (UDisksLinuxDriveObject   *object,
                                       gboolean                  get_hw)
 {
-  GUdevDevice *ret = NULL;
+  UDisksLinuxDevice *ret = NULL;
   /* TODO: actually look at @get_hw */
   if (object->devices != NULL)
     {
@@ -439,14 +438,14 @@ udisks_linux_drive_object_get_block (UDisksLinuxDriveObject   *object,
     {
       GDBusObjectSkeleton *iter_object = G_DBUS_OBJECT_SKELETON (l->data);
       UDisksBlock *block;
-      GUdevDevice *device;
+      UDisksLinuxDevice *device;
       gboolean is_disk;
 
       if (!UDISKS_IS_LINUX_BLOCK_OBJECT (iter_object))
         continue;
 
       device = udisks_linux_block_object_get_device (UDISKS_LINUX_BLOCK_OBJECT (iter_object));
-      is_disk = (g_strcmp0 (g_udev_device_get_devtype (device), "disk") == 0);
+      is_disk = (g_strcmp0 (g_udev_device_get_devtype (device->udev_device), "disk") == 0);
       g_object_unref (device);
 
       if (!is_disk)
@@ -471,11 +470,11 @@ udisks_linux_drive_object_get_block (UDisksLinuxDriveObject   *object,
 
 typedef gboolean (*HasInterfaceFunc)    (UDisksLinuxDriveObject     *object);
 typedef void     (*ConnectInterfaceFunc) (UDisksLinuxDriveObject    *object);
-typedef void     (*UpdateInterfaceFunc) (UDisksLinuxDriveObject     *object,
+typedef gboolean (*UpdateInterfaceFunc) (UDisksLinuxDriveObject     *object,
                                          const gchar    *uevent_action,
                                          GDBusInterface *interface);
 
-static void
+static gboolean
 update_iface (UDisksLinuxDriveObject   *object,
               const gchar              *uevent_action,
               HasInterfaceFunc          has_func,
@@ -484,17 +483,18 @@ update_iface (UDisksLinuxDriveObject   *object,
               GType                     skeleton_type,
               gpointer                  _interface_pointer)
 {
+  gboolean ret = FALSE;
   gboolean has;
   gboolean add;
   GDBusInterface **interface_pointer = _interface_pointer;
 
-  g_return_if_fail (object != NULL);
-  g_return_if_fail (has_func != NULL);
-  g_return_if_fail (update_func != NULL);
-  g_return_if_fail (g_type_is_a (skeleton_type, G_TYPE_OBJECT));
-  g_return_if_fail (g_type_is_a (skeleton_type, G_TYPE_DBUS_INTERFACE));
-  g_return_if_fail (interface_pointer != NULL);
-  g_return_if_fail (*interface_pointer == NULL || G_IS_DBUS_INTERFACE (*interface_pointer));
+  g_return_val_if_fail (object != NULL, FALSE);
+  g_return_val_if_fail (has_func != NULL, FALSE);
+  g_return_val_if_fail (update_func != NULL, FALSE);
+  g_return_val_if_fail (g_type_is_a (skeleton_type, G_TYPE_OBJECT), FALSE);
+  g_return_val_if_fail (g_type_is_a (skeleton_type, G_TYPE_DBUS_INTERFACE), FALSE);
+  g_return_val_if_fail (interface_pointer != NULL, FALSE);
+  g_return_val_if_fail (*interface_pointer == NULL || G_IS_DBUS_INTERFACE (*interface_pointer), FALSE);
 
   add = FALSE;
   has = has_func (object);
@@ -521,11 +521,14 @@ update_iface (UDisksLinuxDriveObject   *object,
 
   if (*interface_pointer != NULL)
     {
-      update_func (object, uevent_action, G_DBUS_INTERFACE (*interface_pointer));
+      if (update_func (object, uevent_action, G_DBUS_INTERFACE (*interface_pointer)))
+        ret = TRUE;
       if (add)
         g_dbus_object_skeleton_add_interface (G_DBUS_OBJECT_SKELETON (object),
                                               G_DBUS_INTERFACE_SKELETON (*interface_pointer));
     }
+
+  return ret;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -541,12 +544,12 @@ drive_connect (UDisksLinuxDriveObject *object)
 {
 }
 
-static void
+static gboolean
 drive_update (UDisksLinuxDriveObject  *object,
               const gchar             *uevent_action,
               GDBusInterface          *_iface)
 {
-  udisks_linux_drive_update (UDISKS_LINUX_DRIVE (object->iface_drive), object);
+  return udisks_linux_drive_update (UDISKS_LINUX_DRIVE (object->iface_drive), object);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -555,17 +558,15 @@ static gboolean
 drive_ata_check (UDisksLinuxDriveObject *object)
 {
   gboolean ret;
-  GUdevDevice *device;
+  UDisksLinuxDevice *device;
 
   ret = FALSE;
   if (object->devices == NULL)
     goto out;
 
-  device = G_UDEV_DEVICE (object->devices->data);
-  if (!g_udev_device_get_property_as_boolean (device, "ID_ATA"))
-    goto out;
-
-  ret = TRUE;
+  device = object->devices->data;
+  if (device->ata_identify_device_data != NULL || device->ata_identify_packet_device_data != NULL)
+    ret = TRUE;
 
  out:
   return ret;
@@ -577,16 +578,18 @@ drive_ata_connect (UDisksLinuxDriveObject *object)
 
 }
 
-static void
+static gboolean
 drive_ata_update (UDisksLinuxDriveObject  *object,
                   const gchar             *uevent_action,
                   GDBusInterface          *_iface)
 {
-  udisks_linux_drive_ata_update (UDISKS_LINUX_DRIVE_ATA (object->iface_drive_ata), object);
+  return udisks_linux_drive_ata_update (UDISKS_LINUX_DRIVE_ATA (object->iface_drive_ata), object);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void apply_configuration (UDisksLinuxDriveObject *object);
+
 static GList *
 find_link_for_sysfs_path (UDisksLinuxDriveObject *object,
                           const gchar            *sysfs_path)
@@ -596,8 +599,8 @@ find_link_for_sysfs_path (UDisksLinuxDriveObject *object,
   ret = NULL;
   for (l = object->devices; l != NULL; l = l->next)
     {
-      GUdevDevice *device = G_UDEV_DEVICE (l->data);
-      if (g_strcmp0 (g_udev_device_get_sysfs_path (device), sysfs_path) == 0)
+      UDisksLinuxDevice *device = l->data;
+      if (g_strcmp0 (g_udev_device_get_sysfs_path (device->udev_device), sysfs_path) == 0)
         {
           ret = l;
           goto out;
@@ -611,86 +614,153 @@ find_link_for_sysfs_path (UDisksLinuxDriveObject *object,
  * udisks_linux_drive_object_uevent:
  * @object: A #UDisksLinuxDriveObject.
  * @action: Uevent action or %NULL
- * @device: A #GUdevDevice device object or %NULL if the device hasn't changed.
+ * @device: A #UDisksLinuxDevice device object or %NULL if the device hasn't changed.
  *
  * Updates all information on interfaces on @drive.
  */
 void
 udisks_linux_drive_object_uevent (UDisksLinuxDriveObject *object,
                                   const gchar            *action,
-                                  GUdevDevice            *device)
+                                  UDisksLinuxDevice      *device)
 {
   GList *link;
+  gboolean conf_changed;
 
   g_return_if_fail (UDISKS_IS_LINUX_DRIVE_OBJECT (object));
-  g_return_if_fail (device == NULL || G_UDEV_IS_DEVICE (device));
+  g_return_if_fail (device == NULL || UDISKS_IS_LINUX_DEVICE (device));
 
   link = NULL;
   if (device != NULL)
-    link = find_link_for_sysfs_path (object, g_udev_device_get_sysfs_path (device));
+    link = find_link_for_sysfs_path (object, g_udev_device_get_sysfs_path (device->udev_device));
   if (g_strcmp0 (action, "remove") == 0)
     {
       if (link != NULL)
         {
-          g_object_unref (G_UDEV_DEVICE (link->data));
+          g_object_unref (UDISKS_LINUX_DEVICE (link->data));
           object->devices = g_list_delete_link (object->devices, link);
         }
       else
         {
           udisks_warning ("Drive doesn't have device with sysfs path %s on remove event",
-                          g_udev_device_get_sysfs_path (device));
+                          g_udev_device_get_sysfs_path (device->udev_device));
         }
     }
   else
     {
       if (link != NULL)
         {
-          g_object_unref (G_UDEV_DEVICE (link->data));
+          g_object_unref (UDISKS_LINUX_DEVICE (link->data));
           link->data = g_object_ref (device);
         }
       else
         {
-          object->devices = g_list_append (object->devices, g_object_ref (device));
+          if (device != NULL)
+            object->devices = g_list_append (object->devices, g_object_ref (device));
         }
     }
 
-  update_iface (object, action, drive_check, drive_connect, drive_update,
-                UDISKS_TYPE_LINUX_DRIVE, &object->iface_drive);
-  update_iface (object, action, drive_ata_check, drive_ata_connect, drive_ata_update,
-                UDISKS_TYPE_LINUX_DRIVE_ATA, &object->iface_drive_ata);
+  conf_changed = FALSE;
+  conf_changed |= update_iface (object, action, drive_check, drive_connect, drive_update,
+                                UDISKS_TYPE_LINUX_DRIVE, &object->iface_drive);
+  conf_changed |= update_iface (object, action, drive_ata_check, drive_ata_connect, drive_ata_update,
+                                UDISKS_TYPE_LINUX_DRIVE_ATA, &object->iface_drive_ata);
+
+  if (conf_changed)
+    apply_configuration (object);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static void
+apply_configuration (UDisksLinuxDriveObject *object)
+{
+  GVariant *configuration = NULL;
+  UDisksLinuxDevice *device = NULL;
+
+  if (object->iface_drive == NULL)
+    goto out;
+
+  configuration = udisks_drive_dup_configuration (object->iface_drive);
+  if (configuration == NULL)
+    goto out;
+
+  device = udisks_linux_drive_object_get_device (object, TRUE /* get_hw */);
+  if (device == NULL)
+    goto out;
+
+  if (object->iface_drive_ata != NULL)
+    {
+      udisks_linux_drive_ata_apply_configuration (UDISKS_LINUX_DRIVE_ATA (object->iface_drive_ata),
+                                                  device,
+                                                  configuration);
+    }
+
+ out:
+  g_clear_object (&device);
+  if (configuration != NULL)
+    g_variant_unref (configuration);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+/* utility routine to blacklist WWNs that are not suitable to use
+ * for identification purposes
+ */
+static gboolean
+is_wwn_black_listed (const gchar *wwn)
+{
+  g_return_val_if_fail (wwn != NULL, FALSE);
+
+  if (g_str_has_prefix (wwn, "0x") || g_str_has_prefix (wwn, "0X"))
+    wwn += 2;
+
+  if (g_ascii_strcasecmp (wwn, "50f0000000000000") == 0)
+    {
+      /* SAMSUNG SP1604N (PATA), see https://bugzilla.redhat.com/show_bug.cgi?id=838691#c4 */
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
+}
+
 static gchar *
 check_for_vpd (GUdevDevice *device)
 {
-  gchar *ret;
+  gchar *ret = NULL;
   const gchar *serial;
   const gchar *wwn;
+  const gchar *path;
 
   g_return_val_if_fail (G_UDEV_IS_DEVICE (device), FALSE);
 
-  ret = NULL;
-
-  /* prefer WWN to serial */
+  /* order of preference: WWN_serial, WWN, serial, path */
   serial = g_udev_device_get_property (device, "ID_SERIAL");
   wwn = g_udev_device_get_property (device, "ID_WWN_WITH_EXTENSION");
-  if (wwn != NULL && strlen (wwn) > 0)
+  path = g_udev_device_get_property (device, "ID_PATH");
+  if (wwn != NULL && strlen (wwn) > 0 && !is_wwn_black_listed (wwn))
     {
-      ret = g_strdup (wwn);
+      if (serial != NULL && strlen (serial) > 0)
+        ret = g_strdup_printf ("%s_%s", wwn, serial);
+      else
+        ret = g_strdup (wwn);
     }
   else if (serial != NULL && strlen (serial) > 0)
     {
       ret = g_strdup (serial);
     }
+  else if (path != NULL && strlen (path) > 0)
+    {
+      ret = g_strdup (path);
+    }
   return ret;
 }
 
 /* <internal>
  * udisks_linux_drive_object_should_include_device:
  * @client: A #GUdevClient.
- * @device: A #GUdevDevice.
+ * @device: A #UDisksLinuxDevice.
  * @out_vpd: Return location for unique ID or %NULL.
  *
  * Checks if we should even construct a #UDisksLinuxDriveObject for @device.
@@ -699,7 +769,7 @@ check_for_vpd (GUdevDevice *device)
  */
 gboolean
 udisks_linux_drive_object_should_include_device (GUdevClient  *client,
-                                                 GUdevDevice  *device,
+                                                 UDisksLinuxDevice  *device,
                                                  gchar       **out_vpd)
 {
   gboolean ret;
@@ -716,19 +786,20 @@ udisks_linux_drive_object_should_include_device (GUdevClient  *client,
    *
    * and we are only interested in the first.
    */
-  if (g_strcmp0 (g_udev_device_get_devtype (device), "disk") != 0)
+  if (g_strcmp0 (g_udev_device_get_devtype (device->udev_device), "disk") != 0)
     goto out;
 
-  vpd = check_for_vpd (device);
+  vpd = check_for_vpd (device->udev_device);
 
   if (vpd == NULL)
     {
       const gchar *name;
       const gchar *vendor;
       const gchar *model;
+      const gchar *dm_name;
       GUdevDevice *parent;
 
-      name = g_udev_device_get_name (device);
+      name = g_udev_device_get_name (device->udev_device);
 
       /* workaround for floppy devices */
       if (g_str_has_prefix (name, "fd"))
@@ -745,8 +816,8 @@ udisks_linux_drive_object_should_include_device (GUdevClient  *client,
         }
 
       /* workaround for missing serial/wwn on VMware */
-      vendor = g_udev_device_get_property (device, "ID_VENDOR");
-      model = g_udev_device_get_property (device, "ID_MODEL");
+      vendor = g_udev_device_get_property (device->udev_device, "ID_VENDOR");
+      model = g_udev_device_get_property (device->udev_device, "ID_MODEL");
       if (g_str_has_prefix (name, "sd") &&
           vendor != NULL && g_strcmp0 (vendor, "VMware") == 0 &&
           model != NULL && g_str_has_prefix (model, "Virtual"))
@@ -756,7 +827,7 @@ udisks_linux_drive_object_should_include_device (GUdevClient  *client,
         }
 
       /* workaround for missing serial/wwn on firewire devices */
-      parent = g_udev_device_get_parent_with_subsystem (device, "firewire", NULL);
+      parent = g_udev_device_get_parent_with_subsystem (device->udev_device, "firewire", NULL);
       if (parent != NULL)
         {
           vpd = g_strdup (name);
@@ -765,13 +836,12 @@ udisks_linux_drive_object_should_include_device (GUdevClient  *client,
         }
 
       /* dm-multipath */
-      const gchar *dm_name;
-      dm_name = g_udev_device_get_sysfs_attr (device, "dm/name");
+      dm_name = g_udev_device_get_sysfs_attr (device->udev_device, "dm/name");
       if (dm_name != NULL && g_str_has_prefix (dm_name, "mpath"))
         {
           gchar **slaves;
           guint n;
-          slaves = udisks_daemon_util_resolve_links (g_udev_device_get_sysfs_path (device), "slaves");
+          slaves = udisks_daemon_util_resolve_links (g_udev_device_get_sysfs_path (device->udev_device), "slaves");
           for (n = 0; slaves[n] != NULL; n++)
             {
               GUdevDevice *slave;
@@ -868,6 +938,14 @@ udisks_linux_drive_object_housekeeping (UDisksLinuxDriveObject  *object,
                            g_dbus_object_get_object_path (G_DBUS_OBJECT (object)));
               g_error_free (local_error);
             }
+          else if (nowakeup && (local_error->domain == UDISKS_ERROR &&
+                                local_error->code == UDISKS_ERROR_DEVICE_BUSY))
+            {
+              /* typically because a "secure erase" operation is pending */
+              udisks_info ("Drive %s is busy",
+                           g_dbus_object_get_object_path (G_DBUS_OBJECT (object)));
+              g_error_free (local_error);
+            }
           else
             {
               g_propagate_prefixed_error (error, local_error, "Error updating SMART data: ");
@@ -881,3 +959,160 @@ udisks_linux_drive_object_housekeeping (UDisksLinuxDriveObject  *object,
  out:
   return ret;
 }
+
+static gboolean
+is_block_unlocked (GList *objects, const gchar *crypto_object_path)
+{
+  gboolean ret = FALSE;
+  GList *l;
+  for (l = objects; l != NULL; l = l->next)
+    {
+      UDisksObject *object = UDISKS_OBJECT (l->data);
+      UDisksBlock *block;
+      block = udisks_object_peek_block (object);
+      if (block != NULL)
+        {
+          if (g_strcmp0 (udisks_block_get_crypto_backing_device (block), crypto_object_path) == 0)
+            {
+              ret = TRUE;
+              goto out;
+            }
+        }
+    }
+ out:
+  return ret;
+}
+
+/**
+ * udisks_linux_drive_object_is_not_in_use:
+ * @object: A #UDisksLinuxDriveObject.
+ * @cancellable: (allow-none): A #GCancellable or %NULL.
+ * @error: A #GError or %NULL.
+ *
+ * Checks if the drive represented by @object is in use and sets
+ * @error if so.
+ *
+ * Returns: %TRUE if @object is not is use, %FALSE if @error is set.
+ */
+gboolean
+udisks_linux_drive_object_is_not_in_use (UDisksLinuxDriveObject   *object,
+                                         GCancellable             *cancellable,
+                                         GError                  **error)
+{
+  GDBusObjectManagerServer *object_manager;
+  const gchar *drive_object_path;
+  gboolean ret = TRUE;
+  GList *objects = NULL;
+  GList *l;
+
+  g_return_val_if_fail (UDISKS_IS_LINUX_DRIVE_OBJECT (object), FALSE);
+  g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  drive_object_path = g_dbus_object_get_object_path (G_DBUS_OBJECT (object));
+
+  object_manager = udisks_daemon_get_object_manager (object->daemon);
+  objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (object_manager));
+
+  /* Visit all block devices related to the drive... */
+  for (l = objects; l != NULL; l = l->next)
+    {
+      GDBusObjectSkeleton *iter_object = G_DBUS_OBJECT_SKELETON (l->data);
+      UDisksBlock *block;
+      UDisksFilesystem *filesystem;
+
+      if (!UDISKS_IS_LINUX_BLOCK_OBJECT (iter_object))
+        continue;
+
+      block = udisks_object_peek_block (UDISKS_OBJECT (iter_object));
+      filesystem = udisks_object_peek_filesystem (UDISKS_OBJECT (iter_object));
+
+      if (g_strcmp0 (udisks_block_get_drive (block), drive_object_path) != 0)
+        continue;
+
+      /* bail if block device is mounted */
+      if (filesystem != NULL)
+        {
+          if (g_strv_length ((gchar **) udisks_filesystem_get_mount_points (filesystem)) > 0)
+            {
+              g_set_error (error,
+                           UDISKS_ERROR,
+                           UDISKS_ERROR_DEVICE_BUSY,
+                           "Device %s is mounted",
+                           udisks_block_get_preferred_device (block));
+              ret = FALSE;
+              goto out;
+            }
+        }
+
+      /* bail if block device is unlocked (LUKS) */
+      if (is_block_unlocked (objects, g_dbus_object_get_object_path (G_DBUS_OBJECT (iter_object))))
+        {
+          g_set_error (error,
+                       UDISKS_ERROR,
+                       UDISKS_ERROR_DEVICE_BUSY,
+                       "Encrypted device %s is unlocked",
+                       udisks_block_get_preferred_device (block));
+          ret = FALSE;
+          goto out;
+        }
+    }
+
+ out:
+  g_list_free_full (objects, g_object_unref);
+  return ret;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+/**
+ * udisks_linux_drive_object_get_siblings:
+ * @object: A #UDisksLinuxDriveObject.
+ *
+ * Gets the siblings for @object, if any.
+ *
+ * Returns: (transfer full) (element-type UDisksLinuxDriveObject): A list of #UDisksLinuxDriveObject
+ *   instances. The returned list should be freed with g_list_free() after each element has been
+ *   freed with g_object_unref().
+ */
+GList *
+udisks_linux_drive_object_get_siblings (UDisksLinuxDriveObject *object)
+{
+  GDBusObjectManagerServer *object_manager;
+  GList *ret = NULL;
+  GList *objects, *l;
+  gchar *sibling_id = NULL;
+
+  if (object->iface_drive == NULL)
+    goto out;
+
+  sibling_id = udisks_drive_dup_sibling_id (object->iface_drive);
+  if (sibling_id == NULL || strlen (sibling_id) == 0)
+    goto out;
+
+  object_manager = udisks_daemon_get_object_manager (object->daemon);
+  objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (object_manager));
+  for (l = objects; l != NULL; l = l->next)
+    {
+      GDBusObjectSkeleton *iter_object = G_DBUS_OBJECT_SKELETON (l->data);
+      UDisksLinuxDriveObject *iter_linux_drive_object;
+
+      if (!UDISKS_IS_LINUX_DRIVE_OBJECT (iter_object))
+        continue;
+
+      iter_linux_drive_object = UDISKS_LINUX_DRIVE_OBJECT (iter_object);
+      if (iter_linux_drive_object->iface_drive != NULL &&
+          g_strcmp0 (udisks_drive_get_sibling_id (iter_linux_drive_object->iface_drive), sibling_id) == 0)
+        {
+          ret = g_list_prepend (ret, g_object_ref (iter_object));
+        }
+    }
+
+ out:
+  ret = g_list_reverse (ret);
+  g_list_foreach (objects, (GFunc) g_object_unref, NULL);
+  g_list_free (objects);
+  g_free (sibling_id);
+  return ret;
+}
+
