@@ -43,6 +43,7 @@
 #include "udisksdaemon.h"
 #include "udiskscleanup.h"
 #include "udisksdaemonutil.h"
+#include "udiskslinuxdevice.h"
 
 /**
  * SECTION:udiskslinuxloop
@@ -119,19 +120,19 @@ udisks_linux_loop_update (UDisksLinuxLoop        *loop,
 {
   UDisksDaemon *daemon;
   UDisksCleanup *cleanup;
-  GUdevDevice *device;
+  UDisksLinuxDevice *device;
   uid_t setup_by_uid;
 
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   cleanup = udisks_daemon_get_cleanup (daemon);
 
   device = udisks_linux_block_object_get_device (object);
-  if (g_str_has_prefix (g_udev_device_get_name (device), "loop"))
+  if (g_str_has_prefix (g_udev_device_get_name (device->udev_device), "loop"))
     {
       gchar *filename;
       gchar *backing_file;
       GError *error;
-      filename = g_strconcat (g_udev_device_get_sysfs_path (device), "/loop/backing_file", NULL);
+      filename = g_strconcat (g_udev_device_get_sysfs_path (device->udev_device), "/loop/backing_file", NULL);
       error = NULL;
       if (!g_file_get_contents (filename,
                                 &backing_file,
@@ -164,13 +165,13 @@ udisks_linux_loop_update (UDisksLinuxLoop        *loop,
       udisks_loop_set_backing_file (UDISKS_LOOP (loop), "");
     }
   udisks_loop_set_autoclear (UDISKS_LOOP (loop),
-                             g_udev_device_get_sysfs_attr_as_boolean (device, "loop/autoclear"));
+                             g_udev_device_get_sysfs_attr_as_boolean (device->udev_device, "loop/autoclear"));
 
   setup_by_uid = 0;
   if (cleanup != NULL)
     {
       udisks_cleanup_has_loop (cleanup,
-                               g_udev_device_get_device_file (device),
+                               g_udev_device_get_device_file (device->udev_device),
                                &setup_by_uid);
     }
   udisks_loop_set_setup_by_uid (UDISKS_LOOP (loop), setup_by_uid);
@@ -238,10 +239,10 @@ handle_delete (UDisksLoop             *loop,
                                                          * requests deleting a loop device previously set up by
                                                          * another user.
                                                          *
-                                                         * Do not translate $(udisks2.device), it's a placeholder and
+                                                         * Do not translate $(drive), it's a placeholder and
                                                          * will be replaced by the name of the drive/device in question
                                                          */
-                                                        N_("Authentication is required to delete the loop device $(udisks2.device)"),
+                                                        N_("Authentication is required to delete the loop device $(drive)"),
                                                         invocation))
         goto out;
     }
@@ -250,6 +251,7 @@ handle_delete (UDisksLoop             *loop,
 
   if (!udisks_daemon_launch_spawned_job_sync (daemon,
                                               NULL, /* UDisksObject */
+                                              "loop-setup", caller_uid,
                                               NULL, /* GCancellable */
                                               0,    /* uid_t run_as_uid */
                                               0,    /* uid_t run_as_euid */
@@ -285,9 +287,9 @@ handle_delete (UDisksLoop             *loop,
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gboolean
-loop_set_autoclear (GUdevDevice  *device,
-                    gboolean      value,
-                    GError      **error)
+loop_set_autoclear (UDisksLinuxDevice  *device,
+                    gboolean            value,
+                    GError            **error)
 {
   gboolean ret = FALSE;
   struct loop_info64 li64;
@@ -296,14 +298,14 @@ loop_set_autoclear (GUdevDevice  *device,
   gint sysfs_autoclear_fd;
   gchar *sysfs_autoclear_path = NULL;
 
-  g_return_val_if_fail (G_UDEV_IS_DEVICE (device), FALSE);
+  g_return_val_if_fail (UDISKS_IS_LINUX_DEVICE (device), FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   /* try writing to the loop/autoclear sysfs file - this may not work
    * since it currently (May 2012) depends on a patch not yet applied
    * upstream (it'll fail in open(2))
    */
-  sysfs_autoclear_path = g_strconcat (g_udev_device_get_sysfs_path (device), "/loop/autoclear", NULL);
+  sysfs_autoclear_path = g_strconcat (g_udev_device_get_sysfs_path (device->udev_device), "/loop/autoclear", NULL);
   sysfs_autoclear_fd = open (sysfs_autoclear_path, O_WRONLY);
   if (sysfs_autoclear_fd > 0)
     {
@@ -326,7 +328,7 @@ loop_set_autoclear (GUdevDevice  *device,
   g_free (sysfs_autoclear_path);
 
   /* if that didn't work, do LO_GET_STATUS, then LO_SET_STATUS */
-  device_file = g_udev_device_get_device_file (device);
+  device_file = g_udev_device_get_device_file (device->udev_device);
   fd = open (device_file, O_RDWR);
   if (fd == -1)
     {
@@ -384,7 +386,7 @@ handle_set_autoclear (UDisksLoop             *loop,
 {
   UDisksObject *object = NULL;
   UDisksDaemon *daemon = NULL;
-  GUdevDevice *device = NULL;
+  UDisksLinuxDevice *device = NULL;
   GError *error = NULL;
   uid_t caller_uid = -1;
 
@@ -416,10 +418,10 @@ handle_set_autoclear (UDisksLoop             *loop,
                                                          * requests changing autoclear on a loop device set up by
                                                          * another user.
                                                          *
-                                                         * Do not translate $(udisks2.device), it's a placeholder and
+                                                         * Do not translate $(drive), it's a placeholder and
                                                          * will be replaced by the name of the drive/device in question
                                                          */
-                                                        N_("Authentication is required to modify the loop device $(udisks2.device)"),
+                                                        N_("Authentication is required to modify the loop device $(drive)"),
                                                         invocation))
         goto out;
     }
