@@ -36,6 +36,7 @@
 #include "udiskscleanup.h"
 #include "udisksdaemonutil.h"
 #include "udisksmountmonitor.h"
+#include "udiskslinuxdevice.h"
 
 /**
  * SECTION:udiskslinuxswapspace
@@ -111,7 +112,7 @@ udisks_linux_swapspace_update (UDisksLinuxSwapspace   *swapspace,
                                UDisksLinuxBlockObject *object)
 {
   UDisksMountMonitor *mount_monitor;
-  GUdevDevice *device;
+  UDisksLinuxDevice *device;
   UDisksMountType mount_type;
   gboolean active;
 
@@ -119,7 +120,7 @@ udisks_linux_swapspace_update (UDisksLinuxSwapspace   *swapspace,
   device = udisks_linux_block_object_get_device (object);
 
   active = FALSE;
-  if (udisks_mount_monitor_is_dev_in_use (mount_monitor, g_udev_device_get_device_number (device), &mount_type) &&
+  if (udisks_mount_monitor_is_dev_in_use (mount_monitor, g_udev_device_get_device_number (device->udev_device), &mount_type) &&
       mount_type == UDISKS_MOUNT_TYPE_SWAP)
     active = TRUE;
   udisks_swapspace_set_active (UDISKS_SWAPSPACE (swapspace), active);
@@ -160,6 +161,8 @@ handle_start (UDisksSwapspace        *swapspace,
   UDisksBaseJob *job;
   GError *error;
   gchar *escaped_device = NULL;
+  uid_t caller_uid;
+  gid_t caller_gid;
 
   error = NULL;
   object = udisks_daemon_util_dup_object (swapspace, &error);
@@ -172,6 +175,20 @@ handle_start (UDisksSwapspace        *swapspace,
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   block = udisks_object_peek_block (object);
 
+  error = NULL;
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon,
+                                               invocation,
+                                               NULL /* GCancellable */,
+                                               &caller_uid,
+                                               &caller_gid,
+                                               NULL,
+                                               &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_error_free (error);
+      goto out;
+    }
+
   if (!udisks_daemon_util_check_authorization_sync (daemon,
                                                     object,
                                                     "org.freedesktop.udisks2.manage-swapspace",
@@ -179,10 +196,10 @@ handle_start (UDisksSwapspace        *swapspace,
                                                     /* Translators: Shown in authentication dialog when the user
                                                      * requests activating a swap device.
                                                      *
-                                                     * Do not translate $(udisks2.device), it's a placeholder and
+                                                     * Do not translate $(drive), it's a placeholder and
                                                      * will be replaced by the name of the drive/device in question
                                                      */
-                                                    N_("Authentication is required to activate swapspace on $(udisks2.device)"),
+                                                    N_("Authentication is required to activate swapspace on $(drive)"),
                                                     invocation))
     goto out;
 
@@ -190,6 +207,7 @@ handle_start (UDisksSwapspace        *swapspace,
 
   job = udisks_daemon_launch_spawned_job (daemon,
                                           object,
+                                          "swapspace-start", caller_uid,
                                           NULL, /* cancellable */
                                           0,    /* uid_t run_as_uid */
                                           0,    /* uid_t run_as_euid */
@@ -235,12 +253,29 @@ handle_stop (UDisksSwapspace        *swapspace,
   UDisksDaemon *daemon;
   UDisksBlock *block;
   UDisksBaseJob *job;
+  uid_t caller_uid;
+  gid_t caller_gid;
   gchar *escaped_device = NULL;
+  GError *error = NULL;
 
   object = UDISKS_OBJECT (g_dbus_interface_get_object (G_DBUS_INTERFACE (swapspace)));
   daemon = udisks_linux_block_object_get_daemon (UDISKS_LINUX_BLOCK_OBJECT (object));
   block = udisks_object_peek_block (object);
 
+  error = NULL;
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon,
+                                               invocation,
+                                               NULL /* GCancellable */,
+                                               &caller_uid,
+                                               &caller_gid,
+                                               NULL,
+                                               &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_error_free (error);
+      goto out;
+    }
+
   /* Now, check that the user is actually authorized to stop the swap space.
    *
    * TODO: want nicer authentication message + special treatment if the
@@ -253,10 +288,10 @@ handle_stop (UDisksSwapspace        *swapspace,
                                                     /* Translators: Shown in authentication dialog when the user
                                                      * requests deactivating a swap device.
                                                      *
-                                                     * Do not translate $(udisks2.device), it's a placeholder and
+                                                     * Do not translate $(drive), it's a placeholder and
                                                      * will be replaced by the name of the drive/device in question
                                                      */
-                                                    N_("Authentication is required to deactivate swapspace on $(udisks2.device)"),
+                                                    N_("Authentication is required to deactivate swapspace on $(drive)"),
                                                     invocation))
     goto out;
 
@@ -264,6 +299,7 @@ handle_stop (UDisksSwapspace        *swapspace,
 
   job = udisks_daemon_launch_spawned_job (daemon,
                                           object,
+                                          "swapspace-stop", caller_uid,
                                           NULL, /* cancellable */
                                           0,    /* uid_t run_as_uid */
                                           0,    /* uid_t run_as_euid */
